from . import globals as G
from .Snapshot import Snapshot
from .Video import Video
import requests, os

class AutomatedTest:
    """
    Helpful representation of a selenium test

    :param testId: the selenium session ID, usually from webdriver
    """
    def __init__(self, testId):
        self.testId = testId
    def setScore(self, score):
        """
        Sets the score for our test in the CBT app

        :param score: should be 'pass', 'fail', or 'unset'. The main module exposes SCORE_PASS, SCORE_FAIL, SCORE_UNSET
        """
        options = {
            'action': 'set_score',
            'score': score
        }
        requests.put(G.api + self.testId, auth=(G.username, G.authkey), data=options)
    def setDescription(self, description):
        """
        Sets the description for our test in the CBT app
        """
        options = {
            'action': 'set_description',
            'description': description
        }
        requests.put(G.api + self.testId, auth=(G.username, G.authkey), data=options)
    def stop(self, score=''):
        # score is optional, will combine setScore and stopTest
        """
        Sends the command to our api to stop the selenium test. Similar to driver.quit()

        :param score: (optional) shortcut for AutomatedTest.setScore
        """
        if score != '':
            self.setScore(score)
        requests.delete(G.api + self.testId, auth=(G.username, G.authkey))
    def takeSnapshot(self, description=''):
        """
        Sends the command to take a snapshot and returns a Snapshot instance

        :param description: (optional) shortcut for Snapshot.setDescription
        :returns: the Snapshot instance for this snapshot
        """
        hash = requests.post(G.api + self.testId + '/snapshots', auth=(G.username, G.authkey)).json()['hash']
        snap = Snapshot(hash, self)
        if description != '':
            snap.setDescription(description)
        return snap
    def getSnapshots(self):
        """
        Gets all snapshots for this test

        :returns: a list of Snapshot objects for this test
        """
        snaps = requests.get(G.api+ self.testId + '/snapshots', auth=(G.username, G.authkey)).json()
        ret = []
        for snap in snaps:
            ret.append(Snapshot(snap['hash'], self))
        return ret
    def saveAllSnapshots(self, directory, prefix='image', useDescription=False):
        """
        Downloads all snapshots for this test into a directory

        :param directory: the directory where the snapshots will be saved
        :param prefix: (optional) defines the prefix used for the filenames
        :param useDescription: (optional) if true, will use the snapshot description instead of the prefix
        """
        snaps = self.getSnapshots()
        self.__makeDirectory(directory)
        for i in range(len(snaps)):
            if useDescription and snaps[i].info['description'] != '':
                img = snaps[i].info['description'] + '.png'
            else:
                img = prefix + str(i) + '.png'
            snaps[i].saveLocally(os.path.join(directory, img))
    def startRecordingVideo(self, description=''):
        """
        Starts recording video for this test

        :param description: shortcut for Video.setDescription
        :returns: the Video instance we started recording
        """
        hash = requests.post(G.api + self.testId + '/videos', auth=(G.username, G.authkey)).json()['hash']
        snap = Video(hash, self)
        if description != '':
            video.setDescription(description)
        return snap
    def getVideos(self):
        """
        Gets all video recordings for this test

        :returns: a list of Video objects for this test
        """
        videos = requests.get(G.api+ self.testId + '/videos', auth=(G.username, G.authkey)).json()
        ret = []
        for video in videos:
            ret.append(Video(video['hash'], self))
        return ret
    def saveAllVideos(self, directory, prefix='video', useDescription=False):
        """
        Downloads all videos for this test into a directory

        :param directory: the directory where the videos will be saved
        :param prefix: (optional) defines the prefix used for the filenames
        :param useDescription: (optional) if true, will use the video description instead of the prefix
        """
        videos = self.getVideos()
        self.__makeDirectory(directory)
        for i in range(len(videos)):
            if useDescription and videos[i].info['description'] != '':
                vid = videos[i].info['description'] + '.mp4'
            else:
                vid = prefix + str(i) + '.mp4'
            videos[i].saveLocally(os.path.join(directory, vid))
    def __makeDirectory(self, dir):
        if not os.path.exists(dir):
            os.mkdir(dir)



from fuzzywuzzy import fuzz
from . import globals as G
import requests

class CapsBuilder:
    """
    Builder for generating selenium capabilities

    All of the with... methods return self for method chaining
    """
    def __init__(self):
        self.capsData = requests.get(G.api + 'browsers').json()
        self.platform = None
        self.browser = None
        self.width = None
        self.height = None
        self.name = None
        self.version = None
        self.recordVideo = None
        self.recordNetwork = None
    def withPlatform(self, platform):
        """
        Sets the platform (OS) the user wants to use. Uses fuzzy string comparison to find best match

        :param platform: a string specifying the platform (eg. Windows 7, Mac 10.13)
        """
        self.platform = platform
        return self
    def withBrowser(self, browser):
        """
        Sets the browser the user wants to use. Uses fuzzy string comparison to find best match

        :param browser: as string specifying the browser (eg. Edge 17, Chrome 55x64)
        """
        self.browser = browser
        return self
    def withResolution(self, width, height):
        """
        Sets the screen size for the test
        """
        self.width = width
        self.height = height
        return self
    def withName(self, name):
        """
        Sets the name that will appear in the web app
        """
        self.name = name
        return self
    def withBuild(self, build):
        """
        Sets the build name in the web app
        """
        # cant be build because of method below
        self.version = build
        return self
    def withRecordVideo(self, bool):
        """
        Records a video for the length of the test
        """
        self.recordVideo = bool
        return self
    def withRecordNetwork(self, bool):
        """
        Records network traffice for the length of the test
        """
        self.recordNetwork = bool
        return self
    def build(self):
        """
        Used to generate the capabilites using any options the user specifies

        :returns: a python dict object that can be passed to the selenium webdriver
        """
        return self.__choose()
    def __bestOption(self, options, target):
        bestRatio = 0
        bestOption = None
        target = target.lower()
        target = target.replace('x64', '64-bit')
        for option in options:
            name = option['name'].lower()
            apiName = option['api_name'].lower()
            ratio = fuzz.partial_ratio(name, target)
            ratio += fuzz.partial_ratio(apiName, target)
            ratio += fuzz.ratio(name, target)
            ratio += fuzz.ratio(apiName, target)
            if ratio > bestRatio:
                bestRatio = ratio
                bestOption = option
        return bestOption
    def __bestBrowserNoPlatform(self, target):
        bestRatio = 0
        bestOption = None
        target = target.lower()
        target = target.replace('x64','64-bit')
        for platform in self.capsData:
            for browser in platform['browsers']:
                name = browser['name'].lower()
                apiName = browser['api_name'].lower()
                ratio = fuzz.partial_ratio(name, target)
                ratio += fuzz.partial_ratio(apiName, target)
                ratio += fuzz.ratio(name, target)
                ratio += fuzz.ratio(apiName, target)
                if ratio > bestRatio:
                    bestRatio = ratio
                    bestOption = browser
        return bestOption
    def __choose(self):
        data = self.capsData
        caps = {
            'username': G.username,
            'password': G.authkey
        }
        platform = None
        browser = None
        if self.platform:
            platform = self.__bestOption(data, self.platform)
            caps.update(platform['caps'])
        if self.browser:
            if platform:
                browser = self.__bestOption(platform['browsers'], self.browser)
            else:
                browser = self.__bestBrowserNoPlatform(self.browser)
            caps.update(browser['caps'])
        if self.width and self.height:
            caps['screenResolution'] = str(self.width) + 'x' + str(self.height)
        if self.name:
            caps['name'] = self.name
        if self.version:
            caps['build'] = self.version
        if self.recordVideo:
            caps['record_video'] = self.recordVideo
        if self.recordNetwork:
            caps['record_network'] = self.recordNetwork
        return caps



import requests

from . import globals as G
from .CapsBuilder import CapsBuilder
from .TestHistoryBuilder import TestHistoryBuilder
from .AutomatedTest import AutomatedTest
from .Snapshot import Snapshot
from .Video import Video

SCORE_PASS = 'pass'
SCORE_FAIL = 'fail'
SCORE_UNSET = 'unset'
hub = "http://hub.crossbrowsertesting.com:80/wd/hub"

def getCapsBuilder():
    """
    Used to get the selenium capability builder

    Generating the CapsBuilder pulls in a large amount of data, so user should not call the constrcutor manually
    """
    G.capsBuilder = G.capsBuilder or CapsBuilder()
    return G.capsBuilder

def login(username, authkey):
    """
    Sets the username and authkey used to make the HTTP requests
    """
    G.username = username
    G.authkey = authkey

def getTestHistoryBuilder():
    """
    Used to get the TestHistoryBuilder

    Can also just call the constructor. Method created to match getCapsBuilder()
    """
    return TestHistoryBuilder()

def getTestHistory(options):
    """
    Returns a python dictionary with the test history, filtering based on the options given.

    :param options: a python dictionary created by the TestHistoryBuilder
    """
    return requests.get(G.api, auth=(G.username, G.authkey), data=options).json()

def getTestFromId(sessid):
    """
    Creates an automated test from the selenium session id

    :param sessid: string for the seleneium session/test id. Should come from WebDriver
    """
    return AutomatedTest(sessid)



from . import globals as G
import requests, sys, time, os, threading

class Video:
    """
    Represents a video recording for a selenium test

    :param hash: the hash for this video, returned by rest api when starting a recording
    :param test: an AutomatedTest object that represents a test currently running
    """
    def __init__(self, hash, test):
        self.hash = hash
        self.testId = test.testId
        self.getInfo()
    def getInfo(self):
        """
        Calls out to api to get updated info for this video

        :returns: a python dict object with all of the info for this video
        """
        self.info = requests.get(G.api + self.testId + '/videos/' + self.hash, auth=(G.username, G.authkey)).json()
        return self.info
    def stopRecording(self):
        """
        Sends the command to stop a video recording
        """
        requests.delete(G.api + self.testId + '/videos/' + self.hash, auth=(G.username, G.authkey))
    def setDescription(self, description):
        """
        Sets the description for this video
        """
        url = G.api + self.testId + '/videos/' + self.hash
        self.info = requests.put(url, auth=(G.username, G.authkey), data={'description':description})
    def saveLocally(self, location):
        """
        Async method to download this video to the location given. Recording will be stopped if active.

        :param location: a string with the location and filename for the video. Should have a .mp4 extension
        """
        t = threading.Thread(target=Video.__saveLocally, args=(self, location))
        t.start()
    def __saveLocally(self, location):
        self.getInfo()
        timeout = 20
        iteration = 1
        #while (iteration < timeout) and (self.info['is_finished'] == False):
        #    time.sleep(2)
        #    iteration += 1
        #    self.getInfo()
        if self.info['is_finished'] == False:
            self.stopRecording
        #iteration = 1
        url = self.info['video']
        r = requests.get(url, stream=True)
        while (iteration < timeout) and (r.status_code != 200):
            time.sleep(2)
            r = requests.get(url, stream=True)
            iteration += 1
        if iteration < timeout:
            path = os.path.split(location)[0]
            if not os.path.exists(path):
                os.mkdir(path)
            with open(location, 'wb') as f:
                for chunk in r.iter_content(chunk_size=128):
                    f.write(chunk)



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='cbthelper',
    version='0.1.2',
    author='Tim Hamilton',
    author_email='timh@crossbrowsertesting.com',
    description='a helper library for cross browser testing\'s selenium api',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/crossbrowsertesting/cbthelper-python',
    packages=setuptools.find_packages(),
    install_requires=[
        'fuzzywuzzy',
    ],
    classifiers=(
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ),
)



from . import globals as G
import requests, sys, time, os, threading

class Snapshot:
    """
    Represents a snapshot for selenium tests

    :param hash: the hash for this image, returned by rest api when taking a screenshot
    :param test: an AutomatedTest object that represents a test currently running
    """
    def __init__(self, hash, test):
        self.hash = hash
        self.testId = test.testId
        self.getInfo()
    def getInfo(self):
        """
        Calls out to api to get updated info for this snapshot

        :returns: a python dict object with all of the info for this Snapshot
        """
        self.info = requests.get(G.api + self.testId + '/snapshots/' + self.hash, auth=(G.username, G.authkey)).json()
        return self.info
    def setDescription(self, description):
        """
        Sets the description for this snapshot
        """
        url = G.api + self.testId + '/snapshots/' + self.hash
        self.info = requests.put(url, auth=(G.username, G.authkey), data={'description':description})
    def saveLocally(self, location):
        """
        Async method to download this snapshot to the location given

        :param location: a string with the location and filename for the image. Should have a .png extension
        """
        t = threading.Thread(target=Snapshot.__saveSnapshot, args=(self, location))
        t.start()
    def __saveSnapshot(self, location):
        url = self.getInfo()['image']
        r = requests.get(url, stream=True)
        timeout = 15
        iteration = 1
        while (iteration < timeout) and (r.status_code != 200):
            r = requests.get(url, stream=True)
            iteration += 1
            time.sleep(1)
        if iteration < timeout:
            path = os.path.split(location)[0]
            if not os.path.exists(path):
                os.mkdir(path)
            with open(location, 'wb') as f:
                for chunk in r.iter_content(chunk_size=128):
                    f.write(chunk)



class TestHistoryBuilder:
    """
    Builder to generate options for getting test history

    All of the with... methods return self for method chaining
    """
    def __init__(self):
        self.options = {}
    def withLimit(self, limit):
        """
        Sets the max number of tests to return
        """
        self.options['num'] = limit
        return self
    def withActive(self, active):
        """
        If set, will only return active or inactive tests

        :param active: boolean value
        """
        self.options['active'] = active
        return self
    def withName(self, name):
        """
        Will only return tests that match the name given
        """
        self.options['name'] = name
        return self
    def withBuild(self, build):
        """
        Will only return tests that match the build given
        """
        self.options['build ']= build
        return self
    def withUrl(self, url):
        """
        Will only return tests that navigate to the same url
        """
        self.options['url'] = url
        return self
    def withScore(self, score):
        """
        Will only return tests with the score specified ('pass', 'fail', 'unset')

        The library contains helpful enums cbthelper.(SCORE_PASS, SCORE_FAIL, SCORE_UNSET)
        """
        self.options['score'] = score
        return self
    def withPlatform(self, platform):
        """
        Will only return tests with the same platform (OS)

        :param platform: string with the platform (eg. 'Windows 10', 'Mac OS 10.13')
        """
        self.options['platform'] = platform
        return self
    def withPlatformType(self, platformType):
        """
        Will only return tests with the same platformType (OS Family)

        :param platformType: string with the platform type (eg. 'Windows', 'Mac', 'Android')
        """
        self.options['platformType'] = platformType
        return self
    def withBrowser(self, browser):
        """
        Will only return tests that used the same browser

        :param browser: a string with the browser name and version: (eg. Chrome 65)
        """
        self.options['browser'] = browser
        return self
    def withBrowserType(self, browserType):
        """
        Will only return tests that used the same browser type

        :param browserType: a string representing the browser family (eg. 'Chrome', 'Edge', 'Safari')
        """
        self.options['browserType'] = browserType
        return self
    def withResolution(self, resolution):
        """
        Will only return tests that used the same resolution

        :param resolution: a string with the form 'WIDTHxHEIGHT' (eg. '1024x768')
        """
        self.options['resolution']= resolution
        return self
    def withStartDate(self, startDate):
        self.options['startDate'] = startDate
        return self
    def withEndDate(self, endDate):
        self.options['endDate'] = endDate
        return self
    def build(self):
        """
        Generates the test history options

        :returns: a python dict to pass to cbthelper.getTestHistory()
        """
        return self.options
